library benchpress.src.reporter.console_reporter;

import "package:angular2/src/facade/lang.dart"
    show print, isPresent, isBlank, NumberWrapper;
import "package:angular2/src/facade/collection.dart"
    show StringMapWrapper, ListWrapper, List;
import "package:angular2/src/facade/async.dart" show Future, PromiseWrapper;
import "package:angular2/src/facade/math.dart" show Math;
import "package:angular2/di.dart" show bind, Binding, OpaqueToken;
import "../statistic.dart" show Statistic;
import "../reporter.dart" show Reporter;
import "../sample_description.dart" show SampleDescription;
import "../measure_values.dart" show MeasureValues;

/**
 * A reporter for the console
 */
class ConsoleReporter extends Reporter {
  num _columnWidth;
  Function _print;
  // TODO(tbosch): use static values when our transpiler supports them
  static OpaqueToken get PRINT {
    return _PRINT;
  }
  // TODO(tbosch): use static values when our transpiler supports them
  static OpaqueToken get COLUMN_WIDTH {
    return _COLUMN_WIDTH;
  }
  // TODO(tbosch): use static values when our transpiler supports them
  static List<Binding> get BINDINGS {
    return _BINDINGS;
  }
  static _lpad(value, columnWidth, [fill = " "]) {
    var result = "";
    for (var i = 0; i < columnWidth - value.length; i++) {
      result += fill;
    }
    return result + value;
  }
  static _formatNum(n) {
    return NumberWrapper.toFixed(n, 2);
  }
  static _sortedProps(obj) {
    var props = [];
    StringMapWrapper.forEach(obj, (value, prop) => props.add(prop));
    props.sort();
    return props;
  }
  List<String> _metricNames;
  ConsoleReporter(this._columnWidth, sampleDescription, this._print) : super() {
    /* super call moved to initializer */;
    this._metricNames = ConsoleReporter._sortedProps(sampleDescription.metrics);
    this._printDescription(sampleDescription);
  }
  _printDescription(sampleDescription) {
    this._print('''BENCHMARK ${ sampleDescription . id}''');
    this._print("Description:");
    var props = ConsoleReporter._sortedProps(sampleDescription.description);
    props.forEach((prop) {
      this._print(
          '''- ${ prop}: ${ sampleDescription . description [ prop ]}''');
    });
    this._print("Metrics:");
    this._metricNames.forEach((metricName) {
      this._print(
          '''- ${ metricName}: ${ sampleDescription . metrics [ metricName ]}''');
    });
    this._print("");
    this._printStringRow(this._metricNames);
    this._printStringRow(this._metricNames.map((_) => "").toList(), "-");
  }
  Future<dynamic> reportMeasureValues(MeasureValues measureValues) {
    var formattedValues = ListWrapper.map(this._metricNames, (metricName) {
      var value = measureValues.values[metricName];
      return ConsoleReporter._formatNum(value);
    });
    this._printStringRow(formattedValues);
    return PromiseWrapper.resolve(null);
  }
  Future<dynamic> reportSample(
      List<MeasureValues> completeSample, List<MeasureValues> validSample) {
    this._printStringRow(this._metricNames.map((_) => "").toList(), "=");
    this._printStringRow(ListWrapper.map(this._metricNames, (metricName) {
      var sample = ListWrapper.map(
          validSample, (measureValues) => measureValues.values[metricName]);
      var mean = Statistic.calculateMean(sample);
      var cv = Statistic.calculateCoefficientOfVariation(sample, mean);
      var formattedMean = ConsoleReporter._formatNum(mean);
      // Note: Don't use the unicode character for +- as it might cause

      // hickups for consoles...
      return NumberWrapper.isNaN(cv)
          ? formattedMean
          : '''${ formattedMean}+-${ Math . floor ( cv )}%''';
    }));
    return PromiseWrapper.resolve(null);
  }
  _printStringRow(parts, [fill = " "]) {
    this._print(ListWrapper.map(parts, (part) {
      var w = this._columnWidth;
      return ConsoleReporter._lpad(part, w, fill);
    }).join(" | "));
  }
}
var _PRINT = new OpaqueToken("ConsoleReporter.print");
var _COLUMN_WIDTH = new OpaqueToken("ConsoleReporter.columnWidth");
var _BINDINGS = [
  bind(ConsoleReporter).toFactory((columnWidth, sampleDescription, print) =>
      new ConsoleReporter(columnWidth, sampleDescription, print), [
    _COLUMN_WIDTH,
    SampleDescription,
    _PRINT
  ]),
  bind(_COLUMN_WIDTH).toValue(18),
  bind(_PRINT).toValue(print)
];
