library benchpress.src.webdriver.firefox_driver_extension;

import "package:angular2/di.dart" show bind, Binding;
import "package:angular2/src/facade/lang.dart" show isPresent, StringWrapper;
import "../web_driver_extension.dart" show WebDriverExtension, PerfLogFeatures;
import "../web_driver_adapter.dart" show WebDriverAdapter;
import "package:angular2/src/facade/async.dart" show Future;

class FirefoxDriverExtension extends WebDriverExtension {
  WebDriverAdapter _driver;
  static List<Binding> get BINDINGS {
    return _BINDINGS;
  }
  bool _profilerStarted;
  FirefoxDriverExtension(this._driver) : super() {
    /* super call moved to initializer */;
    this._profilerStarted = false;
  }
  gc() {
    return this._driver.executeScript("window.forceGC()");
  }
  Future<dynamic> timeBegin(String name) {
    if (!this._profilerStarted) {
      this._profilerStarted = true;
      this._driver.executeScript("window.startProfiler();");
    }
    return this._driver.executeScript("window.markStart(\"" + name + "\");");
  }
  Future<dynamic> timeEnd(String name, [String restartName = null]) {
    var script = "window.markEnd(\"" + name + "\");";
    if (isPresent(restartName)) {
      script += "window.markStart(\"" + restartName + "\");";
    }
    return this._driver.executeScript(script);
  }
  Future<dynamic> readPerfLog() {
    return this._driver
        .executeAsyncScript("var cb = arguments[0]; window.getProfile(cb);");
  }
  PerfLogFeatures perfLogFeatures() {
    return new PerfLogFeatures(render: true, gc: true);
  }
  bool supports(Map<String, dynamic> capabilities) {
    return StringWrapper.equals(
        capabilities["browserName"].toLowerCase(), "firefox");
  }
}
var _BINDINGS = [
  bind(FirefoxDriverExtension).toFactory(
      (driver) => new FirefoxDriverExtension(driver), [WebDriverAdapter])
];
